// Learn cc.Class:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/class.html
//  - [English] http://docs.cocos2d-x.org/creator/manual/en/scripting/class.html
// Learn Attribute:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://docs.cocos2d-x.org/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] https://www.cocos2d-x.org/docs/creator/manual/en/scripting/life-cycle-callbacks.html

cc.Class({
    extends: cc.Component,

    properties: {
        autodestroy: {
            default: false
        },
        destroyParent: {
            default: false
        },
        randomLife: {
            default: false,
            visible() {
                return !this.autodestroy;
            }
        },
        randomLifeMin: {
            default: 1,
            visible() {
                return this.randomLife;
            }
        },
        randomLifeMax: {
            default: 2,
            visible() {
                return this.randomLife;
            }
        },
        timeRandom: {
            default: false
        },
        timeToDestroy: {
            default: 1.0,
            visible() {
                return !this.timeRandom;
            }
        },
        timeToDestroyMin: {
            default: 1.0,
            visible() {
                return this.timeRandom;
            }
        },
        timeToDestroyMax: {
            default: 1.0,
            visible() {
                return this.timeRandom;
            }
        },
        useFade: {
            default: false
        },
        fateTime: {
            default: 1.0
        }
    },

    // LIFE-CYCLE CALLBACKS:
    start() {

        this.isPool = this.node.getComponent('objectPool');
    },

    onLoad () {

        if(this.randomLife) {
            this.timeToDestroy = cc.Random(this.randomLifeMin*10, this.randomLifeMax*10) / 10;
            this.autodestroy = true;
        }
        
        if(this.autodestroy) {

            var t = this.timeToDestroy;
            if(this.timeRandom) {
                this.timeToDestroy = cc.Random(this.timeToDestroyMin*100, this.timeToDestroyMax*100)/100.0;
            }

            cc.tween(this.node).delay(this.timeToDestroy).call(function(evt){

                var _this = this[0];

                if(_this.useFade) {

                    if(_this.timeRandom) {
                        _this.fateTime = cc.Random(_this.timeToDestroyMin*100, _this.timeToDestroyMax*100)/100.0;
                    }
        
                    var fade = cc.fadeOut(_this.fateTime);
                    var call = cc.callFunc(function(){
                        this.onDestroyObject();
                    }, _this, _this);
                    var seq = cc.sequence(fade, call);
                    _this.node.runAction(seq);
                }
                else {
                    _this.onDestroyObject();
                }

            }, [this], this).start();
        }
    },

    onDestroyObject() {

        if(this.isPool !== null) {
            
            this.isPool.manager.put(this.node);
            this.node.active = false;
            cc.log('reinsert in pool');
            return;
        }

        if(this.destroyParent) {

            if(this.node.parent.isValid === true)
                this.node.parent.destroy();

            return;
        }

        if(this.node.isValid === true)
            this.node.destroy();
    },
});
