import { Smallthing} from '__smallthing';

// Learn cc.Class:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/class.html
//  - [English] http://docs.cocos2d-x.org/creator/manual/en/scripting/class.html
// Learn Attribute:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://docs.cocos2d-x.org/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] https://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] https://www.cocos2d-x.org/docs/creator/manual/en/scripting/life-cycle-callbacks.html

cc.Class({
    extends: cc.Component,

    properties: {
        timeVibration: {
            default: 0.0,
            visible() {
                return false;
            } 
        },
        autoVibration: {
            default: false
        },
        autoVibrationTimeMin: {
            default: 0,
            visible() {
                return this.autoVibration;
            }
        },
        autoVibrationTimeMax: {
            default: 1.0,
            visible() {
                return this.autoVibration;
            }
        },
        autoVibrationLength: {
            default: 0.1,
            visible() {
                return this.autoVibration;
            }
        },
        rangeMin: {
            default:-10
        },
        rangeMax: {
            default:10
        },
        destNode: {
            default: null,
            type: cc.Node
        },
        isEnable: {
            default: true
        },
        delay: {
            default: 0
        },
        playAtStart: {
            default: ''
        }
    },

    // LIFE-CYCLE CALLBACKS:

    onLoad () {

    //start () {
        this.animation = this.node.getComponent(cc.Animation);

        this.position = cc.v2(this.node.x, this.node.y);
        this.autoVibrationTimer = 0;
        if(this.autoVibration) {
            this.autoVibrationTimer = cc.RandomFromDistribution.Range(this.autoVibrationTimeMin*10, this.autoVibrationTimeMax*10) / 10;
        }

        if(this.playAtStart != '') {
            this.playAnimation(this.playAtStart);
        }
    },

    vibration(time, destNode) {
        
        if(destNode !== undefined && destNode !== null) {
            this.destNode = destNode;
        }

        this.timeVibration = time;

        if(this.destNode)
        {
            this.position = cc.v2(this.destNode.x, this.destNode.y);
        }
    },

    playAnimation(name) {

        if(this.animation !== null)
            this.animation.play(name);
    },

    update (dt) {

        if(!this.isEnable)
            return;

        if(this.animation !== null && this.animation.currentClip !== null) {

            if(Smallthing.GAMEDATA.isPause === true) {

                if(this.animation.getAnimationState() != null && this.animation.getAnimationState().isPlaying())
                    this.animation.pause();
            }
            else {
                if(this.animation.getAnimationState() != null && this.animation.getAnimationState().isPaused())
                    this.animation.resume();
            }
        }

        if(this.delay > 0) {
            this.delay -= dt;
            return;
        }

        var node = this.node;
        if(this.destNode != null)
            node = this.destNode;

        if(this.autoVibrationTimer >= 0 && this.autoVibration) {
            this.autoVibrationTimer -= dt;
            if(this.autoVibrationTimer < 0) {
                this.vibration(this.autoVibrationLength, null);
            }
        }

        if(this.position !== undefined) {

            if(this.timeVibration >= 0) {

                this.timeVibration -= dt;
                if(this.timeVibration < 0) {
                    if(this.autoVibration) {
                        this.autoVibrationTimer = cc.RandomFromDistribution.Range(this.autoVibrationTimeMin*10, this.autoVibrationTimeMax*10) / 10;
                    }                
                }
                node.x = this.position.x + cc.RandomFromDistribution.Range(this.rangeMin, this.rangeMax);
                node.y = this.position.y + cc.RandomFromDistribution.Range(this.rangeMin, this.rangeMax);
            }
            else {
                node.x = this.position.x;
                node.y = this.position.y;
            }
        }        
    },
});
